---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2013).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity alu is
	generic (NW: INTEGER:= 8); -- word size
	port (A, B: in std_logic_vector (NW-1 downto 0);
		  FS: in std_logic_vector (3 downto 0); -- function select
		  y: out std_logic_vector (NW-1 downto 0);
		  V, C, N, Z: out std_logic); -- V: overflow, C: carry out, N: negative, Z: zero
end alu;

architecture structure of alu is

	component alu_arith
		generic (N: INTEGER:= 8);
		port (A, B : in std_logic_vector (N-1 downto 0);
			  sel : in std_logic_vector (2 downto 0);
			  y : out std_logic_vector (N-1 downto 0);
	          V, C: out std_logic);
	end component;
	
	component alu_logic
		generic (N: INTEGER:= 8);
		port ( A, B: in std_logic_vector (N-1 downto 0);
			   sel: in std_logic_vector (2 downto 0);
			 y: out std_logic_vector (N-1 downto 0));
	end component;

	signal yt, ya, yb: std_logic_vector (NW-1 downto 0);
    signal vt, ct: std_logic;
    
    constant ZERO: std_logic_vector (NW-1 downto 0):= (others => '0');
    
begin

fa: alu_arith generic map (N => NW)
    port map (A => A, B => B, sel => FS (2 downto 0), y => ya, V => vt, C => ct);
	 
fl: alu_logic generic map (N => NW)
    port map (A => A, B => B, sel => FS (2 downto 0), y => yb);
	 
	 with FS(3) select
			yt <= ya when '0',
			     yb when others;

     V <= vt when FS(3) = '0' else '0';
     C <= ct when FS(3) = '0' else '0';
     N <= yt(NW-1); -- negative
          
     --Z <= '1' when yt = ZERO else '0'; -- equal to 0
     -- When FS=1111 (case reserved for JUMP), we don't want N and Z to be activated.
     --   N won't activate (as when we have 1111 the ALU logic just outputs zeros), but Z might:
     process (yt, FS)
     begin
        Z <= '0';
        if FS/="1111" then
            if yt=ZERO then Z <= '1'; end if;     
        end if;
     end process; 
     
     y <= yt;
end structure;

