---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2013).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity alu_logic is
   generic (N: INTEGER:= 8);
	port ( A, B: in std_logic_vector (N-1 downto 0);
		   sel: in std_logic_vector (2 downto 0);
	       y: out std_logic_vector (N-1 downto 0));
end alu_logic;

architecture structure of alu_logic is

    signal srB, slB: std_logic_vector(N-1 downto 0);
    
begin

    srB <= '0'&B(N-1 downto 1);
    slB <= B(N-2 downto 0)&'0';
	with sel select
			y <= A and B   when "000",
			     A or B   when "001",
				 A xor B   when "010",
				 not(A)   when "011",
				 not(B)  when "100",
				 srB   when "101", -- srB, din = 0
				 slB   when "110", -- slB, din = 0
				 (others => '0') when others; -- when "111" -> all outputs are 0.

end structure;

